
#include <windows.h>
#include <winsock.h>
#include <string.h>
#include <ctype.h>

#include "comterm.h"
#include "main.h"
#include "resource.h"
#include "asm.h"
#include "wsa.h"

HINSTANCE hinst;
static HINSTANCE hInstCommDrv;

/*
 * This is a list of the routines that we import from the next-in-line
 * COM Driver.
 */
import importListCom[21] = {
	"CommWriteString",         &lpfnCommWriteString,
	"EnableNotification",      &lpfnEnableNotification,
	"ReactivateOpenCommPorts", &lpfnReactivateOpenCommPorts,
	"ReadCommString",          &lpfnReadCommString,
	"SuspendOpenCommPorts",    &lpfnSuspendOpenCommPorts,
	"cclrbrk",                 &lpfnCclrbrk,
	"cevt",                    &lpfnCevt,
	"cevtGet",                 &lpfnCevtGet,
	"cextfcn",                 &lpfnCextfcn,
	"cflush",                  &lpfnCflush,
	"csetbrk",                 &lpfnCsetbrk,
	"ctx",                     &lpfnCtx,
	"getdcb",                  &lpfnGetdcb,
	"inicom",                  &lpfnInicom,
	"reccom",                  &lpfnReccom,
	"setcom",                  &lpfnSetcom,
	"setque",                  &lpfnSetque,
	"sndcom",                  &lpfnSndcom,
	"stacom",                  &lpfnStacom,
	"trmcom",                  &lpfnTrmcom,
	NULL,                      0 };

/*
 * This is a list of the routines that we import from USER.EXE.  These
 * routines must be obtained this way (instead of resolving their addresses
 * at load time) because we load before USER.EXE.
 */
import importListUser[33] = {
	"BeginPaint",         (FARPROC *) &lpfnBeginPaint,
	"CheckRadioButton",   (FARPROC *) &lpfnCheckRadioButton,
	"CreateWindow",       (FARPROC *) &lpfnCreateWindow,
	"DefWindowProc",      (FARPROC *) &lpfnDefWindowProc,
	"DestroyWindow",      (FARPROC *) &lpfnDestroyWindow,
	"DialogBox",          (FARPROC *) &lpfnDialogBox,
	"DrawIcon",           (FARPROC *) &lpfnDrawIcon,
	"EnableWindow",       (FARPROC *) &lpfnEnableWindow,
	"EndDialog",          (FARPROC *) &lpfnEndDialog,
	"GetDesktopWindow",   (FARPROC *) &lpfnGetDesktopWindow,
	"GetDlgItem",         (FARPROC *) &lpfnGetDlgItem,
	"GetDlgItemInt",      (FARPROC *) &lpfnGetDlgItemInt,
	"GetDlgItemText",     (FARPROC *) &lpfnGetDlgItemText,
	"GetSystemMetrics",   (FARPROC *) &lpfnGetSystemMetrics,
	"GetWindowRect",      (FARPROC *) &lpfnGetWindowRect,
	"IsDlgButtonChecked", (FARPROC *) &lpfnIsDlgButtonChecked,
	"KillTimer",          (FARPROC *) &lpfnKillTimer,
	"LoadCursor",         (FARPROC *) &lpfnLoadCursor,
	"LoadIcon",           (FARPROC *) &lpfnLoadIcon,
	"LoadString",         (FARPROC *) &lpfnLoadString,
	"MessageBox",         (FARPROC *) &lpfnMessageBox,
	"MoveWindow",         (FARPROC *) &lpfnMoveWindow,
	"PostMessage",        (FARPROC *) &lpfnPostMessage,
	"RegisterClass",      (FARPROC *) &lpfnRegisterClass,
	"ScreenToClient",     (FARPROC *) &lpfnScreenToClient,
	"SendDlgItemMessage", (FARPROC *) &lpfnSendDlgItemMessage,
	"SetDlgItemInt",      (FARPROC *) &lpfnSetDlgItemInt,
	"SetDlgItemText",     (FARPROC *) &lpfnSetDlgItemText,
	"SetTimer",           (FARPROC *) &lpfnSetTimer,
	"SetWindowText",      (FARPROC *) &lpfnSetWindowText,
	"WinHelp",            (FARPROC *) &lpfnWinHelp,
	"_wsprintf",          (FARPROC *) &lpfnWsprintf,
	NULL,                 0 };

/*
 * This is a list of routines that we import from KRNL386.EXE.  As it turns
 * out, these routines don't have to be obtained after we load.  They could
 * be resolved at load time, which would eliminate the need for this array.
 */
import importListKernel[9] = {
	"GetPrivateProfileInt",      (FARPROC *) &lpfnGetPrivateProfileInt,
	"GetPrivateProfileString",   (FARPROC *) &lpfnGetPrivateProfileString,
	"GetSystemDirectory",        (FARPROC *) &lpfnGetSystemDirectory,
	"GlobalAlloc",               (FARPROC *) &lpfnGlobalAlloc,
	"GlobalFree",                (FARPROC *) &lpfnGlobalFree,
	"GlobalLock",                (FARPROC *) &lpfnGlobalLock,
	"GlobalUnlock",              (FARPROC *) &lpfnGlobalUnlock,
	"WritePrivateProfileString", (FARPROC *) &lpfnWritePrivateProfileString,
	NULL,                        0 };

/*
 * This is an array of data structures that contain all of the necessary information
 * for each COM Port.
 */
static portInfo ourPorts[MAXCOM];

portInfo *ports[MAXCOM] = { &ourPorts[0],
                            &ourPorts[1],
                            &ourPorts[2],
                            &ourPorts[3],
                            &ourPorts[4],
                            &ourPorts[5],
                            &ourPorts[6],
                            &ourPorts[7],
                            &ourPorts[8] };


/*
 * mainInitPort - This routine is called when the "portInfo" data structure for a
 *                particular COM Port needs to be (re)initialized.  It is only
 *                called from two places:
 *                1. From LibMain, when we initialize ALL of the "portInfo" data
 *                   structures, and
 *                2. From comterm (comterm.c), when we clean up after disconnecting
 *                   from a remote host.
 */
void
mainInitPort (
              portInfo *pIp		/* IN - The address of the portInfo struct to init. */
              )
{

UINT   len;
char   szEntryStr[20] = "DataLenCom";

    pIp->ucOrgDataLen = 0;      /*  Let's assume a 7 bit data length  */

	switch(pIp->sComPort)
		{
		case 0:
			strcat(szEntryStr,"1");
			break;
		case 1:
			strcat(szEntryStr,"2");
			break;
		case 2:
			strcat(szEntryStr,"3");
			break;
		case 3:
			strcat(szEntryStr,"4");
			break;
		case 4:
			strcat(szEntryStr,"5");
			break;
		case 5:
			strcat(szEntryStr,"6");
			break;
		case 6:
			strcat(szEntryStr,"7");
			break;
		case 7:
			strcat(szEntryStr,"8");
			break;
		default:
			strcat(szEntryStr,"9");   // Nine Com ports!?  Thats what the data 
									  // structure looks like, but Com1 - Com8 
									  // is what appears in the Confug utility.
		}

    //  
    // The .ini filename and section are hardcoded here only because the 
	// "lpszIniSection" and "lpszIniFile" variables have not yet been set up. 
    //
    len = (*lpfnGetPrivateProfileString) ("NFSWCOMM", szEntryStr, "", szEntryStr, 80, "PCNFSWIN.INI");

    if(strchr(szEntryStr,(int)'8'))
	    pIp->ucOrgDataLen = 1;


	pIp->sMedia = 0;
	
	pIp->iSock = INVALID_SOCKET;

	pIp->ucMSRShadow = 0;		// Indicate the CTS and DSR are turned off.

	pIp->hwndNotify = 0;
	pIp->usNotifyFlags = 0;

	pIp->lpRxQueue = 0;
	pIp->usRxSize = 0;
	pIp->lpTxQueue = 0;
	pIp->usTxSize = 0;
	pIp->usRxCurLen = 0;
	pIp->usTxCurLen = 0;
	pIp->sRxTrigger = -1;
	pIp->sTxTrigger = 0;

	pIp->usRcvFlag = 3;
	
	pIp->ucIACEcho = 0;			// Initialize to Remote Echo.
	pIp->ucMode = 0;			// Initialize Mode to ascii
	pIp->ucDataLength = pIp->ucOrgDataLen;  // Initialize Data Length to value */
											// read from pcnfswin.ini file.

	pIp->ucIACBuf[0] = 0xFF;	// Set first byte of IAC negotiation strings to IAC.

	pIp->ucIACFlag = 0xff;

	pIp->ucIACSet = 0;

	pIp->iTimer = 0;
	pIp->ucIACSetHold = 0;

	pIp->hwndDlg = 0;
}


/*
 * MainWndProc - This routine is the Windows Procedure for each COM Port session.
 *               It is necessary in order to perform some basic tasks that can only
 *               be performed by a Windows Proc (because COM Driver sessions don't
 *               actually have a session associated with them).
 *
 *               Read the code for details of what tasks are performed.
 */
LONG FAR PASCAL
MainWndProc (
             HWND hWnd,
             UINT Message,
             WPARAM wParam,
             LPARAM lParam
             )
{
	int i;
	portInfo *pIp;
	
	switch (Message) {
	/*
	 * This is the timer procedure that is used during session establishment.
	 * This routine is necessary in order for us to allow the remote host
	 * enough time to perform its IAC negotiation, then to allow us to do our
	 * own negotiation.
	 *
	 * Once the session is established, this routine is no longer needed.
	 */
	case WM_TIMER:
		for (i = 0; i < MAXCOM; i++) {
			if (ports[i]->sComPort == (short) wParam) {
				pIp = ports[i];
				if ((pIp->usRxCurLen == 0) && (++(pIp->iTimer) < 12)) {
					pIp->ucIACSetHold = pIp->ucIACSet;
					break;
				}

				if (pIp->ucIACSetHold != pIp->ucIACSet) {
					pIp->ucIACSetHold = pIp->ucIACSet;
					break;
				}
		
				(*lpfnKillTimer) (hWnd, pIp->uiTimer);
				pIp->usRcvFlag = 0;
				netRecv (pIp);
				if (pIp->hwndDlg != 0)
					(*lpfnEndDialog) (pIp->hwndDlg, IDOK);
				break;
			}
		}

		break;

	/*
	 * This is the code that handles asynchronous notifications from the
	 * Winsock DLL.
	 */
	case ASYNC_SELECT:
		for (i = 0; i < MAXCOM; i++) {
			if (ports[i]->iSock == wParam) {
				switch (WSAGETSELECTEVENT (lParam)) {
					case FD_READ:
						netRecv (ports[i]);
						break;
					case FD_WRITE:
						break;
					case FD_OOB:
						break;
					case FD_CLOSE:
//						comterm (i);
						break;
				}
				
				break;
			}
		}
		
		break;
		
	default:
		return (*lpfnDefWindowProc) (hWnd, Message, wParam, lParam);
	}
	
	return (0L);
}


/*
 * LibMain - This is the main entry-point to the COM Driver at startup.
 *
 *           At this time, it only loads the addresses of the routines in KRNL386.EXE
 *           that we call (and we don't even need to do that.  We could remove this
 *           code, and change all of the (*lpfn...) calls in the rest of the code to
 *           just call the KRNL386.EXE routines directly.) and initializes the
 *           "portInfo" structures.
 */
int CALLBACK
LibMain (
         HINSTANCE hinstCurrent,
         WORD wDataSeg,
         WORD cbHeapSize,
         LPSTR lpszCmdLine
         )
{
	static char *lpszKernel = "KERNEL.EXE";
	HINSTANCE hInstTemp;
	import *iP;
	portInfo *pIp;
	int i;
	
	hinst = hinstCurrent;

	hInstTemp = LoadLibrary (lpszKernel);
	for (i = 0, iP = &importListKernel[i]; iP->lpszName != NULL; i++, iP = &importListKernel[i])
		(FARPROC) *iP->lpfnAddr = GetProcAddress (hInstTemp, iP->lpszName);
	FreeLibrary (hInstTemp);
	
	for (i = 0, pIp = ports[0]; i < MAXCOM; i++, pIp++) {
		pIp->sComPort = i;
		mainInitPort (pIp);
	}
	
	return (1);
}


/*
 * mainFirstTimeInit - This routine will be called when we receive the
 *                     first API call (in ccom.asm).  It is responsible
 *                     for performing the initialization stuff that
 *                     couldn't be done during LibMain, because LibMain
 *                     gets executed before Windows is all the way loaded.
 */
void
mainFirstTimeInit (void)
{
	static char *lpszUser = "USER.EXE";
	HGLOBAL hglb;
	char FAR *lpszOldCommDrv;
	portInfo *pIp;
	HINSTANCE hInstTemp;
	WNDCLASS wc;
	import *iP;
	int i;

	/*
	 * Load USER.EXE, and get the addresses of the routines that we'll be
	 * calling later.
	 */
	hInstTemp = LoadLibrary (lpszUser);
	for (i = 0, iP = &importListUser[i]; iP->lpszName != NULL; i++, iP = &importListUser[i])
		(FARPROC) *iP->lpfnAddr = GetProcAddress (hInstTemp, iP->lpszName);
	FreeLibrary (hInstTemp);

	/*
	 * The following strings are used when loading and evaluating the COM Driver
	 * entries in PCNFSWIN.INI.
	 */
	(*lpfnLoadString) (hinst, IDS_INI_FILE,                lpszIniFile,              sizeof (lpszIniFile));
	(*lpfnLoadString) (hinst, IDS_INI_SECTION,             lpszIniSection,           sizeof (lpszIniSection));
	(*lpfnLoadString) (hinst, IDS_INI_MEDIA_PHYSICAL,      lpszIniMediaPhysical,     sizeof (lpszIniMediaPhysical));
	(*lpfnLoadString) (hinst, IDS_INI_MEDIA_NETWORK,       lpszIniMediaNetwork,      sizeof (lpszIniMediaNetwork));
	(*lpfnLoadString) (hinst, IDS_INI_MEDIA_PHYSICAL_PERM, lpszIniMediaPhysicalPerm, sizeof (lpszIniMediaPhysical));
	(*lpfnLoadString) (hinst, IDS_INI_MEDIA_NETWORK_PERM,  lpszIniMediaNetworkPerm,  sizeof (lpszIniMediaNetwork));

	/*
	 * Allocate a chunk of memory, into which we will build the path to the next-in-line
	 * COM Driver.  After we allocate and lock the memory, get the name of the entry
	 * in PCNFSWIN.INI that contains the name of the next-in-line COM Driver.  If the
	 * driver is named "comm.drv", get the name that the next-in-line COM Driver was
	 * renamed to.  We have to do this because (for reasons I'm not completely sure of,
	 * though I think it has to do with the driver having the same name as the "comm.drv"
	 * entry in SYSTEM.INI) the PC-NFSpro COM Driver can't load another driver that is
	 * named comm.drv.
	 */
	if ((hglb = (*lpfnGlobalAlloc) (GPTR, 1024)) != NULL) {
		lpszOldCommDrv = (*lpfnGlobalLock) (hglb);

		(*lpfnLoadString) (hinst, IDS_INI_NEXT, lpszOldCommDrv, 1024);
		(*lpfnGetPrivateProfileString) (lpszIniSection, lpszOldCommDrv, "", lpszOldCommDrv, 1024, lpszIniFile);
		
		if (strcmpi (lpszOldCommDrv, "comm.drv") == 0) {
			(*lpfnLoadString) (hinst, IDS_INI_RENAMED, lpszOldCommDrv, 1024);
			(*lpfnGetPrivateProfileString) (lpszIniSection, lpszOldCommDrv, "", lpszOldCommDrv, 1024, lpszIniFile);
		}

		hInstCommDrv = LoadLibrary (lpszOldCommDrv);

		for (i = 0, iP = &importListCom[i]; iP->lpszName != NULL; i++, iP = &importListCom[i])
			*iP->lpfnAddr = GetProcAddress (hInstCommDrv, iP->lpszName);

		(*lpfnGlobalUnlock) (hglb);
		(*lpfnGlobalFree) (hglb);
	}

	/*
	 * Register a Windows Class for the COM Driver.  This class will be
	 * used by the "cominit" routine to create a Window Procedure, which will
	 * be responsible for fielding and processing Timer events and Asynchronous
	 * Winsock events.
	 */
	szDriverClass = "NFSWCommClass";
	wc.style = 0;
	wc.lpfnWndProc = MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hinst;
	wc.hIcon = NULL;
	wc.hCursor = (*lpfnLoadCursor) (NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = szDriverClass;
	(*lpfnRegisterClass) (&wc);

	/*
	 * The name of our Help File.
	 */
	(*lpfnLoadString) (hinst, IDS_HELP_FILE, lpszHelpFile, sizeof (lpszHelpFile));

	/*
	 * Load and initialize the names of the PCNFSWIN.INI entries for each
	 * COM Port.
	 */
	for (i = 0, pIp = ports[0]; i < MAXCOM; i++, pIp++) {
		(*lpfnLoadString) (hinst, IDS_INI_MEDIA, szString, sizeof (szString));
		(*lpfnWsprintf) (pIp->szIniMedia, szString, pIp->sComPort + 1);
		(*lpfnLoadString) (hinst, IDS_INI_HOST,  szString, sizeof (szString));
		(*lpfnWsprintf) (pIp->szIniHost,  szString, pIp->sComPort + 1);
		(*lpfnLoadString) (hinst, IDS_INI_TERM,  szString, sizeof (szString));
		(*lpfnWsprintf) (pIp->szIniTerm,  szString, pIp->sComPort + 1);
		(*lpfnLoadString) (hinst, IDS_INI_PORT,  szString, sizeof (szString));
		(*lpfnWsprintf) (pIp->szIniPort,  szString, pIp->sComPort + 1);
	}
}
