;
; Copyright (c) 1993, 1994 by Sun Microsystems, Inc.
;

.xlist
include cmacros.inc
include comdev.inc

include structs.inc
.list

;;
;; This is our Data Segment.
;;
sBegin  Data

;
; If the user doesn't specify a Terminal Type (from the Redirection Dialog Box),
; send this string to the remote host during Terminal Type negotiation.
;
unknown_msg	db	'UNKNOWN'

sEnd	Data


;;
;; This is our Code Segment.
;;
sBegin  Code
assumes cs,Code
assumes ds,Data

;;
;; External procedures.
;;
	externNP	netSendBuf


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; net_iac_ck - This routine processes incoming IAC traffic.
;
; Supported protocols:
;	ECHO
;	SPGA
;	BINARY
;	TERMINAL-TYPE 
;
; IN:  BX = A NEAR pointer to the "portInfo" structure for this COM Port.
;      [BX].(*lpRxQueue + usRxCurLen) is the address of the first IAC
;           data in the Receive Queue.
;      CX = The length of the new data after the first IAC char,
;           including the IAC char.
;
; OUT: None.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	public	net_iac_ck

net_iac_ck	proc	near
	xor	dx, dx			; Initialize no IAC flags.
					;
	les	di, [bx].lpRxQueue	; Set ES:DI to point to the first
	add	di, [bx].usRxCurLen	;    byte of the new data.
;
; Check for IAC support
;
	or	cx, cx			; If there's data available, jump.
	jz	net_iacDone		; Otherwise, leave.
					;
	mov	al, es:[di]		; Get the next byte of data and adjust
	inc	di			;    the pointer and length.
	dec	cx			;
					;
	cmp	al, 0ffh		; Check for IAC support (255),
	je	iac3			;    and jump if it is.
	cmp	[bx].cIncomp, 1		; If we have an incomplete protocol #1
	je	iac6			;    go get the next part if IAC.
	cmp	[bx].cIncomp, 2		; If we have an incomplete protocol #2
	je	net_iac_disp		;    go get the next part if IAC.
	cmp	[bx].cIncomp, 3		; If we have an incomplete protocol #3
	je	net_iac_sb_disp		;    go get the next part if IAC.
					;
	jmp	net_iacDone		; ERROR!!!!!
					;
iac3:	or	cx, cx			; If we're NOT outta data, jump.
	jnz	iac6			;    Otherwise, set incomplete
	mov	[bx].cIncomp, 1		;    protocol #1 and leave.
	jmp	net_iacDone		;
;
; If this is a double IAC, then echo the second IAC to the terminal.  That's
; how TELNET sends an IAC as user-destined data (IAC IAC).
;
iac6:	mov	al, es:[di]		; Get the next byte of data and adjust
	inc	di			;    the pointer and length.
	dec	cx			;
					;
	mov	[bx].cIncomp, 0		; Reset the incomplete protocal flag.
	cmp	al, 0ffh		; If this isn't a double IAC, jump.
	jnz	iac6a			;
	inc	[bx].usRxCurLen		;
	jmp	net_iacDone		;
;
; IAC Two character function dispatcher
;
iac6a:	cmp	al, 0f9h		;check if GA, 249
	jz	net_iacResetACK		;skip if it is
					;
	cmp	al, 0f1h		;check if NOP, 241
	jz	net_iacResetACK		;skip if it is
					;
	mov	[bx].cIACFunct, al	;save IAC function
					;
	or	cx, cx			;
	jnz	net_iac_disp		;
	mov	[bx].cIncomp,2		;set incomplete protocal #2
	jmp	net_iacDone		;exit, indicate iac found
;
;IAC Three character function dispatcher
;
net_iac_disp:				;
	mov	al, es:[di]		; Get the next byte of data and adjust
	inc	di			;    the pointer and length.
	dec	cx			;
	cmp	[bx].cIACFunct, 0fah	;check if SB, 250
	jz	net_iac_sb		;skip if is
					;
	mov	[bx].cIncomp, 0		;reset incomplete protocal
	cmp	[bx].cIACFunct, 0fbh	;check if WILL, 251
	je	net_iac_will		;skip if is
	cmp	[bx].cIACFunct, 0fch	;check if WONT, 252
	jz	net_iac_wont		;skip if is
	cmp	[bx].cIACFunct, 0fdh	;check if DO, 253
	jz	net_iac_do		;skip if is
	cmp	[bx].cIACFunct, 0feh	;check if DONT, 254
	jz	net_iac_dont		;skip if is
;
;Assume Two character IAC, skip around it and continue
;
	jmp	net_iacDone		;output character
;
;IAC Handshaking Processes for REQUEST ACKNOWLEDGEMENT from the host
;
;================================
;WILL request processes		|
;================================
;
;WILL ECHO process 
;
net_iac_will:				;
	cmp	al, 1			;check for receive echo
	jne	net_iac_will5		;skip if not
	mov	[bx].ucIACEcho, 0	;set for remote echo ON
	mov	dl, 1			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get DO ECHO state	
	or	[bx].ucIACSet, 1	;set DO ECHO state
	test	[bx].ucIACFlag, 1	;check if they initiated req
	jz	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge DO if different
;
	and	[bx].ucIACFlag, 07fh	;set DUPLEX negotiations
	test	dh, 1			;check if DO remote ECHO set
	jz	net_iac_setdo		;ackn DO remote ECHO
	jmp	net_iacResetACK		;skip if it is, DO already set
;
;WILL SPGA process 
;
net_iac_will5:				;
	cmp	al, 3			;check for SPGA
	jne	net_iac_will10		;skip if not
	mov	dl, 2			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get DO SPGA state	
	or	[bx].ucIACSet, 2		;set DO SPGA state
	test	[bx].ucIACFlag, 2	;check if they initiated req
	jz	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge DO if different
;
	test	dh, 2			;check if DO SPGA set
	jz	net_iac_setdo		;ackn DO SPGA
	jmp	net_iacResetACK		;skip if it is, DO already set
;
;WILL BINARY process 
;
net_iac_will10:				;
	or	al, al			;check for BINARY
	jnz	net_iac_setdont		;skip if not, default DONT ackn
					;
	mov	dl, 8			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get DO BINARY state	
	or	[bx].ucIACSet, 8		;set DO BINARY state
	test	[bx].ucIACFlag, 8	;check if they initiated req
	jz	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge DO if different
;
	test	dh, 8			;check if DO BINARY set
	jz	net_iac_setdo		;ackn DO BINARY
	jmp	net_iacResetACK		;skip if it is, DO already set

;================================
;WONT request processes		|
;================================
;
;WONT ECHO process
;
net_iac_wont:				;
	cmp	al, 1			;check for remote echo
	jne	net_iac_wont5		;skip if not
	mov	[bx].ucIACEcho, 1	;set remote echo off
	mov	dl, 1			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get DO ECHO state	
	and	[bx].ucIACSet, 0feh	;set DONT ECHO state
	test	[bx].ucIACFlag, 1	;check if they initiated req
	jnz	net_iac_wont3		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge DONT if different
;
net_iac_wont3:				;
	and	[bx].ucIACFlag, 07fh	;set DUPLEX negotiations
	test	dh, 1			;check if DO remote ECHO
	jnz	net_iac_wont4		;
	jmp	net_iacResetACK		;skip if not, DONT already set
					;
net_iac_wont4:				;
	jmp	net_iac_setdont		;ackn DONT remote ECHO
;
;WONT SPGA process
;
net_iac_wont5:				;
	cmp	al, 3			;check for SPGA
	jne	net_iac_wont10		;skip if not
	mov	dl, 2			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get DO SPGA state	
	and	[bx].ucIACSet, 0fdh	;set DONT SPGA state
	test	[bx].ucIACFlag, 2	;check if they initiated req
	jnz	net_iac_wont8		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge DONT if different
;
net_iac_wont8:				;
	test	dh, 2			;check if DO SPGA set
	jnz	net_iac_wont9		;
	jmp	net_iacResetACK		;skip if not, DONT already set
					;
net_iac_wont9:				;
	jmp	net_iac_setdont		;ackn DONT SPGA
;
;WONT BINARY process
;
net_iac_wont10:				;
	or	al, al			;check for BINARY
	jz	net_iac_wont11		;
	jmp	net_iacResetACK		;skip if not, default no ackn
					;
net_iac_wont11:				;
	mov	dl, 8			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get DO BINARY state	
	and	[bx].ucIACSet, 0f7h	;set DONT BINARY state
	test	[bx].ucIACFlag, 8	;check if they initiated req
	jnz	net_iac_wont12		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge DONT if different
;
net_iac_wont12:				;
	test	dh, 8			;check if DO BINARY set
	jnz	net_iac_wont13		;
	jmp	net_iacResetACK		;skip if not, DONT already set
					;
net_iac_wont13:				;
	jmp	net_iac_setdont		;ackn DONT BINARY

;================================
;DO request processes		|
;================================
;
;DO ECHO not supported
;
net_iac_do:				;
	cmp	al, 1			;check for send echo
	jnz	net_iac_do2		;
	jmp	net_iac_setwont		;send WONT echo, not supported
;
;DO SPGA process
;
net_iac_do2:				;
	cmp	al, 3			;check for SPGA
	jne	net_iac_do10		;skip if not
	mov	dl, 4			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get WILL SPGA state	
	or	[bx].ucIACSet, 4		;set WILL SPGA state
	test	[bx].ucIACFlag, 4	;check if they initiated req
	jnz	net_iac_do8		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
;
;There request, acknowledge WILL if different
;
net_iac_do8:				;
	test	dh, 4			;check if WILL SPGA set
	jz	net_iac_do9		;
	jmp	net_iacResetACK		;skip if it is, WILL already set
					;
net_iac_do9:				;
	jmp	net_iac_setwill		;ackn WILL SPGA
;
;DO BINARY process
;
net_iac_do10:				;
	or	al, al			;check for BINARY
	jne	net_iac_doTermType	;skip if not
	mov	[bx].ucDataLength, 1	;set binary data lentgh
	mov [bx].ucMode,1	; Set mode to binary
	mov	dl, 10h			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get WILL BINARY state	
	or	[bx].ucIACSet, 10h	;set WILL BINARY state
	test	[bx].ucIACFlag, 10h	;check if they initiated req
	jnz	net_iac_do11		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
;
;Acknowledge WILL if different
;
net_iac_do11:				;
	test	dh, 10h			;check if WILL BINARY set
	jz	net_iac_do12		;
	jmp	net_iacResetACK		;skip if it is, WILL already set
					;
net_iac_do12:				;
	jmp	net_iac_setwill		;ackn WILL BINARY
;
; DO TERMINAL-TYPE process
;
net_iac_doTermType:			;
	cmp	al, 18h			; If this is NOT TERMINAL-TYPE,
	jnz	net_iac_setwont		;    jump.
					;
	mov	dl, 20h			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get WILL TERMINAL-TYPE state	
	or	[bx].ucIACSet, 20h	;set WILL TERMINAL-TYPE state
	test	[bx].ucIACFlag, 20h	; If the host initiated the request,
	jz	net_iacResetACK		;    don't send an ACK.
;
; There request, acknowledge WILL if different
;
	test	dh, 20h			;check if WILL TERMINAL-TYPE set
	jnz	net_iacResetACK		;skip if it is, WILL already set
	jmp	net_iac_setwill		;ackn WILL TERMINAL-TYPE

;================================
;DONT request processes		|
;================================
;
;DONT ECHO not supported
;
net_iac_dont:				;
	cmp	al, 1			;check for remote echo
	jnz	net_iac_dont2		;
	jmp	net_iacResetACK		;skip if it is, not supported
;
;DONT SPGA process
;
net_iac_dont2:				;
	cmp	al, 3			;check for SPGA
	jne	net_iac_dont10		;skip if not, default no ackn
	mov	dl, 4			;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get WILL SPGA state	
	and	[bx].ucIACSet, 0fbh	;set WONT SPGA state
	test	[bx].ucIACFlag, 4	;check if they initiated req
	jnz	net_iac_dont4		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
					;
net_iac_dont4:				;
;
;There request, acknowledge WONT if different
;
	test	dh, 4			;check if WILL SPGA set
	jnz	net_iac_dont5		;
	jmp	net_iacResetACK		;skip if not, WONT already set
					;
net_iac_dont5:				;
	jmp	net_iac_setwont		;ackn WONT SPGA
;
;DONT BINARY process
;
net_iac_dont10:				;
	or	al, al			;check for BINARY
	jz	net_iac_dont10a		;
	jmp	net_iacResetACK		;skip if not, default no ackn
					;
net_iac_dont10a:			;

	push ax					;
	mov [bx].ucMode,0			;Set ascii mode
	mov al, [bx].ucOrgDataLen   ;Get Data Len read from INI file.
	mov	[bx].ucDataLength, al	;set Data len accordingly.
	pop ax						;
	mov	dl, 010h		;set acknowledgement flag
	mov	dh, [bx].ucIACSet	;get WILL BINARY state	
	and	[bx].ucIACSet, 0efh	;set WONT BINARY state
	test	[bx].ucIACFlag, 10h	;check if they initiated req
	jnz	net_iac_dont11		;
	jmp	net_iacResetACK		;skip if not, we did, no ack
;
;Acknowledge WONT if different
;
net_iac_dont11:				;
	test	dh, 10h			;check if WILL BINARY set
	jnz	net_iac_dont12		;
	jmp	net_iacResetACK		;skip if not, WONT already set
					;
net_iac_dont12:				;
	jmp	net_iac_setwont		;ackn WONT BINARY

;================================
;SB request processes		|
;================================
;
;SB TERMINAL-TYPE process
;
net_iac_sb:				;
	cmp	al, 18h			;check for TERMINAL-TYPE, 24
	jnz	net_iacResetACK		;skip if not, default no ackn
;
;Get next IAC character
;
net_iac_sb_disp:			;
	or	cx, cx			;
	jnz	net_iac_sb5		;
	mov	[bx].cIncomp, 3		;set incomplete protocal #3
	jmp	net_iacDone		;exit, indicate iac found
					;
net_iac_sb5:				;
	mov	al, es:[di]		; Get the next byte of data and adjust
	inc	di			;    the pointer and length.
	dec	cx			;
					;
	cmp	al, 0f0h		;check for SE code, 240
	jne	net_iac_sb_disp		;skip if not
	mov	[bx].cIncomp, 0		;reset incomplete protocal
;
; Set initial IAC data to output buffer.  Note that we're pushing the length
; of the remaining unprocessed data (CX) onto the stack.  This value will
; remain on the stack until we get to net_iac_opt5.
;
	push	cx			; Save the remaining char count.
					;
	push	si			;save si.
	push	di			;save current buffer position
	push	es			;save es
					;
	push	ds			;set es to our data segment.
	pop	es			;
	mov	[bx].ucIACBuf + 1, 0fah	;set SB function, 250
	mov	word ptr [bx].ucIACBuf + 2, 0018h ;set TERMINAL-TYPE option and
					;IS code, 24 & 0
	mov	di, bx			;
	add	di, cIACSub		;set destination address
;
; If the user specified a custom terminal type, always send that type
; for our response to the TERMINAL-TYPE request (no matter how many
; times the host asks for it).
;
	cmp	[bx].sTermTypeLen, 0	; If we aren't a custom terminal
	jz	net_iac_getTermType	;    type, jump.
					;
	mov	si, bx			; Get the terminal-type and
	add	si, szTermType		;    the length of the type.
	mov	cx, [bx].sTermTypeLen	;
					;
	mov	dx, 6			; Calculate the length of the
	add	dx, cx			;    response (6 byte header and
	jmp	short net_iac_sb10	;    trailer), then go send it.
;
; If the user didn't specify a terminal type, set the TERMINAL-TYPE
; subnegotiation response to UNKNOWN.
;
net_iac_getTermType:			;
	mov	dx, 13			;set unknown output count
	mov	si, offset DGROUP:unknown_msg;set message address
	mov	cx, 7			;set string size
;
; Copy terminal type to output buffer
;
net_iac_sb10:				;
	rep	movsb			;copy terminal type to buffer
	mov	word ptr [di], 0f0ffh	;set IAC SE codes, 255 & 240 
	mov	cx, dx			;set output count
	mov	dl, 0			;set to not change iac flags
					;
	pop	es			;restore es
	pop	di			;restore di
	pop	si			;restore current buffer position
	jmp	short net_iac_opt5	;send subnegotiation request
;
;Send IAC jump table
;
net_iac_setwill:			;
	mov	[bx].ucIACBuf + 1, 0fbh	;send WILL, 251
	jmp	short net_iac_opt	;set option
					;
net_iac_setwont:			;
	mov	[bx].ucIACBuf + 1, 0fch	;send WONT, 252
	jmp	short net_iac_opt	;set option
					;
net_iac_setdo:				;
	mov	[bx].ucIACBuf + 1, 0fdh	;send DO, 253
	jmp	short net_iac_opt	;set option
					;
net_iac_setdont:			;
	mov	[bx].ucIACBuf + 1, 0feh	;send DONT, 254
					;
net_iac_opt:				;
	mov	[bx].ucIACBuf + 2, al	;send option
	push	cx			;save remaining character count
	mov	cx, 3			;set output count
;
; At this point, the length of the remaining unprocessed data in the
; Receive Queue is on the top of the stack.
;
net_iac_opt5:				;
	push	cx			; Save CX, ES, and DI.
	push	es			;
	push	di			;
					;
	mov	di, bx			; Set ES:DI to address of the IACBuf
	add	di, ucIACBuf		;    for this COM Port.
	push	ds			;
	pop	es			;
	or	[bx].ucIACFlag, dl	; Reset the acknowledgement flag.
	call	netSendBuf		;
					;
	pop	di			; Restore DI, ES, and CX.
	pop	es			;
	pop	cx			;
					;
	pop	cx			; Restore the remaining char count.
	jmp	short net_iacDone	;
					;
net_iacResetACK:			;
	or	[bx].ucIACFlag, dl	;reset acknowledgement flag
;
; We're done.  ES:DI currently points to the first unprocessed byte of data
; in the Receive Queue (i.e. the first byte after the IAC sequence we just
; processed).  Shift all of the data that follows the just-processed IAC
; data down, so that it replaces the just-processed IAC data.  This will
; throw away the just-processed IAC data, and allow CX to continue to
; reflect the amount of new data at the end of the Receive Queue.
;
net_iacDone:				;
	push	cx			; Save CX and DS.
	push	ds			;
					;
	mov	si, di			; Set ??:SI to the remaining data.
					;
	les	di, [bx].lpRxQueue	; Set ES:DI to point just past the
	add	di, [bx].usRxCurLen	;    the already processed user data.
					;
	push	es			; Set DS:DI to point to the first byte
	pop	ds			;    of the remaining unprocessed data.
	cld				;
	rep	movsb			;
					;
	pop	ds			; Restore DS and CX.
	pop	cx			;
	ret
net_iac_ck	endp


;---------------------------------------------
;Check if IAC ECHO or BINARY changed by user |
;---------------------------------------------
;
;If user changed ECHO, attempt to set full duplex if remote echo,
;Else if local echo, attempt half duplex. 
;
	public	iac_user_ck

iac_user_ck	proc	near
	push	ax			;
;
;ECHO/SPGA CHECK
;If negotiations for ECHO & SPGA not set, start timer to reset
;
	mov	al, [bx].ucIACFlag	;get flags
	and	al, 7h			; Clear the DUPLEX & BINARY bits.
	cmp	al, 7h			;check acknowledgements
	je	iac_user_echo		;skip if all there
;
;Reset all acknowledgements
;
	test	[bx].cIACTrouble, 1	;check ECHO/SPGA problems before
	jne	iac_user_bin		;skip if there was
	or	[bx].cIACTrouble, 1	;set trouble bit
	or	[bx].ucIACFlag, 7h	;reset acknowledgements
;
; Check to see if the user changed the ECHO state. 
;
iac_user_echo:				;
	mov	al, [bx].ucIACSet	;get iac echo state
	and	al, 1			;set echo state only
	xor	al, 1			;...
	cmp	[bx].ucIACEcho, al	;check if echo change by user
	je	iac_user_bin		;skip if not
;
;Terminal user changed local/remote ECHO state
;Send ECHO IAC
;
	mov	al, 01h			;set DO ECHO FLAG
	mov	ah, 1			;set ECHO option
	cmp	[bx].ucIACEcho, 0	;check DO remote echo
	je	iac_user10		;skip if it is
	call	iac_dont		;send DONT ECHO
	jmp	short iac_user12	;
					;
iac_user10:				;
	call	iac_do			;send DO ECHO
					;
iac_user12:				;
	xor	[bx].ucIACEcho, 1	;temp reset till DO ack
;
;Attempt full or half duplex based on echo setting
;
iac_user20:				;
	cmp	[bx].ucIACEcho, 1	;check DO remote echo,(reversed)
	jne	iac_user_bin		;skip if not
;
;REMOTE ECHO set
;Attempt to WILL and DO SPGA
;
	call	set_spga		;attempt WILL and DO SPGA
;
;BINARY USER CHECK
;If negotiations for ECHO & SPGA not set, start timer to reset
;
iac_user_bin:				;
	mov	al, [bx].ucIACFlag	;get flags
	and	al, 18h			;reject DUPLEX, ECHO & SPGA bits
	cmp	al, 18h			;check acknowledgements
	je	iac_user_bin1		;skip if all there
;
;Reset all acknowledgements
;
	test	[bx].cIACTrouble, 2	;check BINARY problems before
	jne	iac_user_ret		;skip if there was
	or	[bx].cIACTrouble, 2	;set trouble bit
	or	[bx].ucIACFlag, 18h	;reset acknowledgements
;
;Check if user changed BINARY state
;
iac_user_bin1:				;
	mov	al, 0			;set ascii
	test	[bx].ucIACSet, 10h	;check WILL BINARY state only
	je	iac_user30		;skip if not binary
	inc	al			;set binary state
					;
iac_user30:				;
	cmp	[bx].ucMode, al	;check if binary change by user
	je	iac_user_ret	;skip if not
;
;Terminal user changed BINARY state
;Send BINARY IAC
;
	mov	ah, 0			;set BINARY option
	mov	al, 10h			;set WILL BINARY FLAG
;
;If not BINARY send WONT/DONT
;
	test	[bx].ucIACSet, 10h	;check WILL BINARY state only
;	cmp	[bx].ucDataLength, 1	;check WILL binary
	jne	iac_user35		;skip if not
;
;BINARY set. Send WILL/DO
;
	call	iac_will		;send WILL BINARY
	test	[bx].ucIACSet, 08h	;check DO BINARY state only
	jne	iac_user37		;skip if binary
	mov	al, 08h			;set DO BINARY FLAG
	call	iac_do			;send DO BINARY
	jmp	short iac_user37	;
;
;No BINARY set. Send WONT/DONT
;
iac_user35:				;
	call	iac_wont		;send WONT BINARY
	test	[bx].ucIACSet, 08h	;check DO BINARY state only
	je	iac_user37		;skip if not binary
	mov	al, 08h			;set DO BINARY FLAG
	call	iac_dont		;send DONT BINARY
					;
iac_user37:				
;	xor	[bx].ucDataLength, 1	;temp reset till WILL ack
					;
iac_user_ret:				;
	clc				;
	pop	ax			;
	ret				;
iac_user_ck	endp


;-------------------------------------
;Check if IAC ECHO changed by server |
;-------------------------------------
;If server changed ECHO, attempt to set full duplex if remote echo,
;Else if local echo, attempt half duplex. 

	public	iac_serv_ck

iac_serv_ck	proc	near
	push	ax			;
;
;Check if in DO ECHO negotiations
;Check if in DO or WILL SPGA negotiations
;
	mov	al, [bx].ucIACFlag	;get flags
	and	al, 7			; Clear the DUPLEX & BINARY bits.
	cmp	al, 7			;check ECHO & SPGA ack
	jne	iac_serv_ret		;skip if not, in negotiations
;
;All IAC negotiations acknowledged
;Check if duplex negotiations open
;
	test	[bx].ucIACFlag, 80h	;check duplex acknowledgement
	jne	iac_serv_ret		;skip if is, negotiations not
					;requested
;
;Loop counter to attempt negotiations after 5 rcv calls
;
;jriggs	dec	[bx].cSPGACnt		;
;jriggs	jne	iac_serv_ret		;skip if not done yet
;jriggs	mov	[bx].cSPGACnt, 5	;reset count
	or	[bx].ucIACFlag, 80h	;set DUPLEX acknowledgement
;
;Check if remote or local echo
;
	test	[bx].ucIACSet, 1	;check if DO ECHO set
	je	iac_serv_ret		;skip if not, local set
;
;REMOTE ECHO set
;Attempt to WILL and DO SPGA
;
	call	set_spga		;attempt WILL and DO SPGA
;
;LOCAL ECHO set
;Attempt to WONT and DONT SPGA
;
iac_serv_ret:				;
	clc				;
	pop	ax			;
	ret				;
iac_serv_ck	endp


set_spga	proc	near
	push	ax			;
;
;Check if in DO or WILL SPGA negotiations
;
	test	[bx].ucIACFlag, 2	;check DO SPGA acknowledgement
	je	set_spga_ret		;skip if not, in negotiations
	test	[bx].ucIACFlag, 4	;check WILL SPGA acknowledgement
	je	set_spga_ret		;skip if not, in negotiations
					;
set_spga10:				;
	mov	ah, 3			;set SPGA option
	test	[bx].ucIACSet, 2	;check if DO SPGA set
	jne	set_spga20		;skip if it is
;
;DO SPGA not set
;Send DO SPGA
;
	mov	al, 02h			;set DO SPGA FLAG
	call	iac_do			;send DO SPGA
;
;DO SPGA set
;If WILL SPGA not set, Send WILL SPGA
;
set_spga20:				;
	test	[bx].ucIACSet, 4	;check if WILL SPGA set
	jne	set_spga_ret		;skip if it is, done
	mov	al, 04h			;set WILL SPGA FLAG
	call	iac_will		;send WILL SPGA
					;
set_spga_ret:				;
	clc				;
	pop	ax			;
	ret				;
set_spga	endp


;On input
;
; al = iac flag bit
; ah = iac option

iac_will	proc	near
	mov	[bx].ucIACBuf + 1, 251	;WILL function
	jmp	short iac_will10	;


iac_wont	proc	near
	mov	[bx].ucIACBuf + 1, 252	;WONT function
	jmp	short iac_will10	;


iac_do		proc	near
	mov	[bx].ucIACBuf + 1, 253	;DO function
	jmp	short iac_will10	;


iac_dont	proc	near
	mov	[bx].ucIACBuf + 1, 254	;DONT function
					;
iac_will10:				;
	push	ax			;
	push	cx			;
	push	es			;
	push	di			;
	xor	al, 0ffh		;compliment bits for logical AND
	and	[bx].ucIACFlag, al	;reset flag request
					;
	mov	di, bx			;
	add	di, ucIACBuf		;
	push	ds			;
	pop	es			;
	mov	byte ptr [di], 0ffh	;set IAC command
	mov	byte ptr [di + 2], ah	;set option
	mov	cx, 3			;set size
	call	netSendBuf		;buffer data
	pop	di			;
	pop	es			;
	pop	cx			;
	pop	ax			;
	ret				;
iac_dont	endp
iac_do		endp
iac_wont	endp
iac_will	endp

sEnd   Code
end
